#!/bin/bash

# Test an IP address for validity:
# Usage:
#      valid_ip IP_ADDRESS
#      if [[ $? -eq 0 ]]; then echo good; else echo bad; fi
#   OR
#      if valid_ip IP_ADDRESS; then echo good; else echo bad; fi
#
function valid_ip()
{
    local ip=$1
    local stat=1

    if [[ $ip =~ ^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$ ]]; then
        OIFS=$IFS
        IFS='.'
        ip=($ip)
        IFS=$OIFS
        [[ ${ip[0]} -le 255 && ${ip[1]} -le 255 \
            && ${ip[2]} -le 255 && ${ip[3]} -le 255 ]]
        stat=$?
    fi
    return $stat
}

# Request the flash drive from the user
# Usage:
# 	get_flash_drive
function get_flash_drive() {
	FLASH_DEVICES=`ls /dev/sd* | grep -v '[0-9]'`;
	echo "Choose which system device is the flash disk to image:" 1>&2;
	echo "${FLASH_DEVICES}" | nl -s ")  "  1>&2;
	read FLASH_ID;
	if ! [[ "${FLASH_ID}" =~ ^[0-9]+$ ]] ; then
		echo "You must enter a number corresponding to a flash device." 1>&2;
		exit;
	fi
	declare -a FLASH_DEVICES=(${FLASH_DEVICES});
	FLASH_ID=$((${FLASH_ID}-1));
	if [[ "${FLASH_ID}" -lt "0" ]] || [[ "${FLASH_ID}" -ge "${#FLASH_DEVICES[@]}" ]]; then
		echo "Invalid number: choose enter a number corresponding to a flash device." 1>&2;
		exit;
	fi
	FLASH_DEVICE="${FLASH_DEVICES[${FLASH_ID}]}";
	echo "Preparing to use device '${FLASH_DEVICE}'..." 1>&2;
	echo "${FLASH_DEVICE}";
}

if [ "`which git`" = "" ]; then
	echo "Please run sudo apt-get install git";
	exit;
fi

echo "Please enter the server hostname or IP:";
read SERVER_ADDRESS;
if valid_ip ${SERVER_ADDRESS}; then
	SERVER_IP="${SERVER_ADDRESS}";
else
	SERVER_IP=`host ${SERVER_ADDRESS} | awk '{print $4;}'`;
	if ! valid_ip ${SERVER_IP}; then
		echo "Not a resolvable hostname or valid IP address.";
		exit;
	fi
fi
echo "Using server address '${SERVER_IP}'...";

MAIN_FOLDER="${PWD}/gpxe-ticc";
mkdir -p "${MAIN_FOLDER}";
cd "${MAIN_FOLDER}";
cat > boot-script.gpxe <<BOOTSCRIPT
#!gpxe

dhcp net0
set 209:string pxelinux.cfg/gpxe
set 210:string http://${SERVER_IP}/gpxe/
chain \${210:string}pxelinux.0
BOOTSCRIPT
cat > tg3.diff <<BROADCOMDRIVER
diff --git a/src/drivers/net/tg3.c b/src/drivers/net/tg3.c
index 20027b2..81cf667 100644
--- a/src/drivers/net/tg3.c
+++ b/src/drivers/net/tg3.c
@@ -3410,6 +3410,7 @@ PCI_ROM(0x14e4, 0x16a7, "tg3-5703X",       "Broadcom Tigon 3 5703X", 0),
 PCI_ROM(0x14e4, 0x16a8, "tg3-5704S",       "Broadcom Tigon 3 5704S", 0),
 PCI_ROM(0x14e4, 0x16c6, "tg3-5702A3",      "Broadcom Tigon 3 5702A3", 0),
 PCI_ROM(0x14e4, 0x16c7, "tg3-5703A3",      "Broadcom Tigon 3 5703A3", 0),
+PCI_ROM(0x14e4, 0x167d, "tg3-5751M",       "Broadcom Tigon 3 5751M", 0),
 PCI_ROM(0x14e4, 0x170d, "tg3-5901",        "Broadcom Tigon 3 5901", 0),
 PCI_ROM(0x14e4, 0x170e, "tg3-5901_2",      "Broadcom Tigon 3 5901_2", 0),
 PCI_ROM(0x1148, 0x4400, "tg3-9DXX",        "Syskonnect 9DXX", 0),
BROADCOMDRIVER
git clone git://git.etherboot.org/scm/gpxe.git;
cd "${MAIN_FOLDER}/gpxe";
#git reset --hard v1.0.1
patch -p1 < ../tg3.diff;
rm ../tg3.diff;
cd "${MAIN_FOLDER}/gpxe/src/";

echo "Choose which type of image to build:" 1>&2;
echo "	1) USB Flash Image"  1>&2;
echo "	2) GRUB-Loadable Image" 1>&2;
read IMAGETYPE;
case "${IMAGETYPE}" in
	1)
		make EMBEDDED_IMAGE="${MAIN_FOLDER}/boot-script.gpxe" bin/gpxe.usb;
		CONTINUE="y";
		while [ "${CONTINUE}" = "y" ]; do
			FLASH_DEVICE=`get_flash_drive`;
			sudo dd if="${MAIN_FOLDER}/gpxe/src/bin/gpxe.usb" of=${FLASH_DEVICE};
			sudo eject "${FLASH_DEVICE}";
			echo "Image another drive? [y/N]" 1>&2;
			read CONTINUE;
		done;
		;;
	2)
		make EMBEDDED_IMAGE="${MAIN_FOLDER}/boot-script.gpxe" bin/gpxe.lkrn;
		cp "${MAIN_FOLDER}/gpxe/src/bin/gpxe.lkrn" "${MAIN_FOLDER}/../gpxe.lkrn";
		;;
	*)
		echo "Choose a valid installation option (either 1 or 2)..." 1>&2;
		;;
esac

cd "${MAIN_FOLDER}/..";
rm -Rf "${MAIN_FOLDER}";
